package com.yonge.cooleshow.common.config;

import com.beust.jcommander.internal.Maps;
import com.yonge.toolset.base.enums.BaseEnum;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;
import org.springframework.stereotype.Component;

import java.util.Map;

@Component
public class EnumConverterFactory implements ConverterFactory<Object, BaseEnum<?, ?>> {

	private static final Map<Class<?>, Converter> CONVERTERS = Maps.newHashMap();

	@Override
	public <T extends BaseEnum<?, ?>> Converter<Object, T> getConverter(Class<T> targetType) {

		/*return source ->  {
			T ret = null;

			for (T t : targetType.getEnumConstants()) {

				// 枚举类型为字符串
				if (t.getCode().equals(source)) {
					ret = t;
				}
			}
			return ret;
		};*/

		Converter<Object, T> converter = CONVERTERS.get(targetType);
		if (converter == null) {
			converter = new EnumConverter<>(targetType);
			CONVERTERS.put(targetType, converter);
		}
		return converter;
	}

	/**
	 * 根据code获取枚举
	 * @param targetType 枚举类型
	 * @param source code
	 * @return 枚举
	 * @param <T> 枚举类型
	 */
	public static <T extends BaseEnum<?, ?>> T getEnum(Class<T> targetType, String source) {
		for (T constant : targetType.getEnumConstants()) {
			if (source.equals(String.valueOf(constant.getCode()))) {
				return constant;
			}
		}
		return null;
	}

}
